# Generated from MySqlBaseParser.g4 by ANTLR 4.9.2
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .MySqlBaseParser import MySqlBaseParser
else:
    from MySqlBaseParser import MySqlBaseParser

# This class defines a complete generic visitor for a parse tree produced by MySqlBaseParser.

class MySqlBaseParserVisitor(ParseTreeVisitor):

    # Visit a parse tree produced by MySqlBaseParser#root.
    def visitRoot(self, ctx:MySqlBaseParser.RootContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#sqlStatements.
    def visitSqlStatements(self, ctx:MySqlBaseParser.SqlStatementsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#sqlStatement.
    def visitSqlStatement(self, ctx:MySqlBaseParser.SqlStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#emptyStatement.
    def visitEmptyStatement(self, ctx:MySqlBaseParser.EmptyStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#dmlStatement.
    def visitDmlStatement(self, ctx:MySqlBaseParser.DmlStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#selectStatement.
    def visitSelectStatement(self, ctx:MySqlBaseParser.SelectStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#insertStatement.
    def visitInsertStatement(self, ctx:MySqlBaseParser.InsertStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#updateStatement.
    def visitUpdateStatement(self, ctx:MySqlBaseParser.UpdateStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#deleteStatement.
    def visitDeleteStatement(self, ctx:MySqlBaseParser.DeleteStatementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#insertStatementValue.
    def visitInsertStatementValue(self, ctx:MySqlBaseParser.InsertStatementValueContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#selectElements.
    def visitSelectElements(self, ctx:MySqlBaseParser.SelectElementsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#selectStarElement.
    def visitSelectStarElement(self, ctx:MySqlBaseParser.SelectStarElementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#selectColumnElement.
    def visitSelectColumnElement(self, ctx:MySqlBaseParser.SelectColumnElementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#tableSource.
    def visitTableSource(self, ctx:MySqlBaseParser.TableSourceContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#tableName.
    def visitTableName(self, ctx:MySqlBaseParser.TableNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#tableAlias.
    def visitTableAlias(self, ctx:MySqlBaseParser.TableAliasContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#fullId.
    def visitFullId(self, ctx:MySqlBaseParser.FullIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#fullColumnName.
    def visitFullColumnName(self, ctx:MySqlBaseParser.FullColumnNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#dottedId.
    def visitDottedId(self, ctx:MySqlBaseParser.DottedIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#uidList.
    def visitUidList(self, ctx:MySqlBaseParser.UidListContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#uid.
    def visitUid(self, ctx:MySqlBaseParser.UidContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#updatedElement.
    def visitUpdatedElement(self, ctx:MySqlBaseParser.UpdatedElementContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#whereClause.
    def visitWhereClause(self, ctx:MySqlBaseParser.WhereClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#expressions.
    def visitExpressions(self, ctx:MySqlBaseParser.ExpressionsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#comparisonPredicate.
    def visitComparisonPredicate(self, ctx:MySqlBaseParser.ComparisonPredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#expressionAtomPredicate.
    def visitExpressionAtomPredicate(self, ctx:MySqlBaseParser.ExpressionAtomPredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#inPredicate.
    def visitInPredicate(self, ctx:MySqlBaseParser.InPredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#betweenPredicate.
    def visitBetweenPredicate(self, ctx:MySqlBaseParser.BetweenPredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#notExpression.
    def visitNotExpression(self, ctx:MySqlBaseParser.NotExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#likePredicate.
    def visitLikePredicate(self, ctx:MySqlBaseParser.LikePredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#nullPredicate.
    def visitNullPredicate(self, ctx:MySqlBaseParser.NullPredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#logicalExpression.
    def visitLogicalExpression(self, ctx:MySqlBaseParser.LogicalExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#predicate.
    def visitPredicate(self, ctx:MySqlBaseParser.PredicateContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#specificFunctionCall.
    def visitSpecificFunctionCall(self, ctx:MySqlBaseParser.SpecificFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#scalarFunctionCall.
    def visitScalarFunctionCall(self, ctx:MySqlBaseParser.ScalarFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#scalarFunctionName.
    def visitScalarFunctionName(self, ctx:MySqlBaseParser.ScalarFunctionNameContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#simpleFunctionCall.
    def visitSimpleFunctionCall(self, ctx:MySqlBaseParser.SimpleFunctionCallContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#functionArgs.
    def visitFunctionArgs(self, ctx:MySqlBaseParser.FunctionArgsContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#functionArg.
    def visitFunctionArg(self, ctx:MySqlBaseParser.FunctionArgContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#functionNameBase.
    def visitFunctionNameBase(self, ctx:MySqlBaseParser.FunctionNameBaseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#comparisonOperator.
    def visitComparisonOperator(self, ctx:MySqlBaseParser.ComparisonOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#logicalOperator.
    def visitLogicalOperator(self, ctx:MySqlBaseParser.LogicalOperatorContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#keywordsCanBeId.
    def visitKeywordsCanBeId(self, ctx:MySqlBaseParser.KeywordsCanBeIdContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#constant.
    def visitConstant(self, ctx:MySqlBaseParser.ConstantContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#nullLiteral.
    def visitNullLiteral(self, ctx:MySqlBaseParser.NullLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#realLiteral.
    def visitRealLiteral(self, ctx:MySqlBaseParser.RealLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#bitString.
    def visitBitString(self, ctx:MySqlBaseParser.BitStringContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#booleanLiteral.
    def visitBooleanLiteral(self, ctx:MySqlBaseParser.BooleanLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#stringLiteral.
    def visitStringLiteral(self, ctx:MySqlBaseParser.StringLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#decimalLiteral.
    def visitDecimalLiteral(self, ctx:MySqlBaseParser.DecimalLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#hexadecimalLiteral.
    def visitHexadecimalLiteral(self, ctx:MySqlBaseParser.HexadecimalLiteralContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#orderByClause.
    def visitOrderByClause(self, ctx:MySqlBaseParser.OrderByClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#orderByExpression.
    def visitOrderByExpression(self, ctx:MySqlBaseParser.OrderByExpressionContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#limitClause.
    def visitLimitClause(self, ctx:MySqlBaseParser.LimitClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#lockClause.
    def visitLockClause(self, ctx:MySqlBaseParser.LockClauseContext):
        return self.visitChildren(ctx)


    # Visit a parse tree produced by MySqlBaseParser#parameterMarker.
    def visitParameterMarker(self, ctx:MySqlBaseParser.ParameterMarkerContext):
        return self.visitChildren(ctx)



del MySqlBaseParser


#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @author jsbxyyx

# https://github.com/jsbxyyx/antlr4-mysql

from antlr4 import *

from .autogen.MySqlBaseLexer import MySqlBaseLexer
from .autogen.MySqlBaseParser import MySqlBaseParser
from .autogen.MySqlBaseParserVisitor import MySqlBaseParserVisitor

debug = False


def log(value, args=None):
    if debug:
        print(value, args)


class MySqlBase:

    @classmethod
    def parser(cls, sql):
        lexer = MySqlBaseLexer(InputStream(sql))
        stream = CommonTokenStream(lexer)
        parser = MySqlBaseParser(stream)
        return parser

    @classmethod
    def parserSQLStatement(cls, sql):
        parser = cls.parser(sql)
        return parser.sqlStatements().sqlStatement()


class LockClause:
    def __init__(self):
        self.forUpdate = None
        pass


class NullPredicate:
    def __init__(self):
        self.predicate = None
        self.NOT = None
        pass


class LikePredicate:
    def __init__(self):
        self.predicate1 = None
        self.NOT = None
        self.predicate2 = None
        pass


class LogicalExpression:
    def __init__(self):
        self.expression1 = None
        self.logicalOperator = None
        self.expression2 = None


class NotExpression:
    def __init__(self):
        self.expression = None


class BetweenPredicate:
    def __init__(self):
        self.predicate1 = None
        self.NOT = None
        self.predicate2 = None
        self.predicate3 = None
        pass


class UpdateElement:
    def __init__(self):
        self.column = None
        self.value = None
        pass


class WhereClause:
    def __init__(self):
        self.expression = None


class Expression:
    def __init__(self):
        pass


class InPredicate:
    def __init__(self):
        self.predicate = None
        self.NOT = None
        self.expressions = list()


class ComparisonPredicate:
    def __init__(self):
        self.predicate1 = None
        self.comparisonOperator = None
        self.predicate2 = None


class PredicateAtom:
    def __init__(self):
        pass


class FullColumnName(PredicateAtom):
    def __init__(self):
        super(FullColumnName, self).__init__()
        self.owner = None
        self.value = None


class Constant(PredicateAtom):
    def __init__(self):
        super(Constant, self).__init__()


class FunctionCall(PredicateAtom):
    def __init__(self):
        super(FunctionCall, self).__init__()
        self.function_name = None
        self.args_list = None


class ParameterMarker(Constant):
    def __init__(self, index, name, value):
        self.index = index
        self.name = name
        self.value = value


class StringLiteral(Constant):
    def __init__(self, value):
        self.value = value[1:-1]


class DecimalLiteral(Constant):
    def __init__(self, value):
        self.value = int(value)


class HexadecimalLiteral(Constant):
    def __init__(self, value):
        self.value = value


class BooleanLiteral(Constant):
    def __init__(self, value):
        self.value = bool(value)


class RealLiteral(Constant):
    def __init__(self, value):
        self.value = float(value)


class NullLiteral(Constant):
    def __init__(self):
        self.value = None


class BitString(Constant):
    def __init__(self, value):
        self.value = value


class Default:
    def __init__(self):
        self.value = 'DEFAULT'


class SelectStatement:
    def __init__(self):
        self.selectElements = None
        self.tableSource = None
        self.where = None
        self.orderBy = None
        self.limit = None
        self.lock = None
        pass


class InsertStatement:
    def __init__(self):
        self.ignore = None
        self.tableSource = None
        self.columns = None
        self.value_list = None


class UpdateStatement:
    def __init__(self):
        self.tableSource = None
        self.updatedElement = None
        self.where = None
        self.orderBy = None
        self.limit = None


class DeleteStatement:
    def __init__(self):
        self.tableAlias = None
        self.tableSource = None
        self.where = None
        self.orderBy = None
        self.limit = None


class OrderByClause:
    def __init__(self):
        self.orderByExpressions = None


class OrderByExpression:
    def __init__(self):
        self.expression = None
        self.ASC = None


class LimitClause:
    def __init__(self):
        self.offset = None
        self.limit = None


class TableSource:
    def __init__(self):
        self.tableName = None
        self.tableAlias = None


class MysqlStatementVisitor(MySqlBaseParserVisitor):
    def __init__(self):
        self.parameter_marker_index = -1

    def visitSelectStatement(self, ctx: MySqlBaseParser.SelectStatementContext):
        log('visitSelectStatement')
        ss = SelectStatement()
        ss.selectElements = self.visit(ctx.selectElements())
        ss.tableSource = self.visit(ctx.tableSource())
        if ctx.whereClause() is not None:
            ss.where = self.visit(ctx.whereClause())
        if ctx.limitClause() is not None:
            ss.limit = self.visit(ctx.limitClause())
        if ctx.orderByClause() is not None:
            ss.orderBy = self.visit(ctx.orderByClause())
        if ctx.lockClause() is not None:
            ss.lock = self.visit(ctx.lockClause())
        return ss

    def visitInsertStatement(self, ctx: MySqlBaseParser.InsertStatementContext):
        log('visitInsertStatement')
        ins = InsertStatement()
        ins.ignore = ctx.IGNORE() is not None
        ins.tableSource = self.visit(ctx.tableSource())
        if ctx.uidList() is not None:
            ins.columns = self.visit(ctx.uidList())
        ins.value_list = self.visit(ctx.insertStatementValue())
        return ins

    def visitUpdateStatement(self, ctx: MySqlBaseParser.UpdateStatementContext):
        log('visitUpdateStatement')
        us = UpdateStatement()
        us.tableSource = self.visit(ctx.tableSource())
        elements = []
        ues = ctx.updatedElement()
        for i in range(len(ues)):
            elements.append(self.visit(ues[i]))
        us.updatedElement = elements
        if ctx.whereClause() is not None:
            us.where = self.visit(ctx.whereClause())
        if ctx.orderByClause() is not None:
            us.orderBy = self.visit(ctx.orderByClause())
        if ctx.limitClause() is not None:
            us.limit = self.visit(ctx.limitClause())
        return us

    def visitDeleteStatement(self, ctx: MySqlBaseParser.DeleteStatementContext):
        log('visitDeleteStatement')
        ds = DeleteStatement()
        if ctx.tableAlias() is not None:
            ds.tableAlias = self.visit(ctx.tableAlias())
        ds.tableSource = self.visit(ctx.tableSource())
        ds.where = self.visit(ctx.whereClause())
        if ctx.orderByClause() is not None:
            ds.orderBy = self.visit(ctx.orderByClause())
        if ctx.LIMIT() is not None:
            ds.limit = self.visit(ctx.decimalLiteral())
        return ds

    def visitInsertStatementValue(self, ctx: MySqlBaseParser.InsertStatementValueContext):
        log('visitInsertStatementValue')
        values = []
        eps = ctx.expressions()
        for i in range(len(eps)):
            values.append(self.visit(eps[i]))
        return values

    def visitSelectElements(self, ctx: MySqlBaseParser.SelectElementsContext):
        log('visitSelectElements')
        elements = []
        ses = ctx.selectElement()
        for i in range(len(ses)):
            elements.append(self.visit(ses[i]))
        return elements

    def visitSelectStarElement(self, ctx: MySqlBaseParser.SelectStarElementContext):
        log('visitSelectStarElement')
        return ctx.getText()

    def visitSelectColumnElement(self, ctx: MySqlBaseParser.SelectColumnElementContext):
        log('visitSelectColumnElement')
        return self.visit(ctx.fullColumnName())

    def visitTableSource(self, ctx: MySqlBaseParser.TableSourceContext):
        log('visitTableSource')
        ts = TableSource()
        ts.tableName = self.visit(ctx.tableName())
        if ctx.tableAlias() is not None:
            ts.tableAlias = self.visit(ctx.tableAlias())
        return ts

    def visitTableName(self, ctx: MySqlBaseParser.TableNameContext):
        log('visitTableName')
        return ctx.getText()

    def visitTableAlias(self, ctx: MySqlBaseParser.TableAliasContext):
        log('visitTableAlias')
        return ctx.getText()

    def visitFullId(self, ctx: MySqlBaseParser.FullIdContext):
        log('visitFullId')
        return ctx.getText()

    def visitFullColumnName(self, ctx: MySqlBaseParser.FullColumnNameContext):
        log('visitFullColumnName')
        fcn = FullColumnName()
        fcn.value = self.visit(ctx.uid())
        if len(ctx.dottedId()) > 0:
            fcn.owner = self.visit(ctx.uid())
            fcn.value = self.visit(ctx.dottedId(len(ctx.dottedId()) - 1))
        return fcn

    def visitDottedId(self, ctx: MySqlBaseParser.DottedIdContext):
        log('visitDottedId')
        return ctx.getText()

    def visitUidList(self, ctx: MySqlBaseParser.UidListContext):
        log('visitUidList')
        uid_list = []
        uids = ctx.uid()
        for i in range(len(uids)):
            uid_list.append(self.visit(uids[i]))
        return uid_list

    def visitUid(self, ctx: MySqlBaseParser.UidContext):
        log('visitUid')
        return ctx.getText()

    def visitUpdatedElement(self, ctx: MySqlBaseParser.UpdatedElementContext):
        log('visitUpdatedElement')
        ue = UpdateElement()
        ue.column = self.visit(ctx.fullColumnName())
        if ctx.DEFAULT() is not None:
            ue.value = Default()
        else:
            ue.value = self.visit(ctx.expression())
        return ue

    def visitWhereClause(self, ctx: MySqlBaseParser.WhereClauseContext):
        log('visitWhereClause')
        wc = WhereClause()
        wc.expression = self.visit(ctx.expression())
        return wc

    def visitExpressions(self, ctx: MySqlBaseParser.ExpressionsContext):
        log('visitExpressions')
        expressions = []
        eps = ctx.expression()
        for i in range(len(eps)):
            expressions.append(self.visit(eps[i]))
        return expressions

    def visitComparisonPredicate(self, ctx: MySqlBaseParser.ComparisonPredicateContext):
        log('visitComparisonPredicate')
        cp = ComparisonPredicate()
        cp.predicate1 = self.visit(ctx.predicate(0))
        cp.comparisonOperator = self.visit(ctx.comparisonOperator())
        cp.predicate2 = self.visit(ctx.predicate(1))
        return cp

    def visitInPredicate(self, ctx: MySqlBaseParser.InPredicateContext):
        log('visitInPredicate')
        ip = InPredicate()
        ip.predicate = self.visit(ctx.predicate())
        ip.NOT = ctx.NOT() is not None
        ip.expressions = self.visit(ctx.expressions())
        return ip

    def visitExpressionAtomPredicate(self, ctx: MySqlBaseParser.ExpressionAtomPredicateContext):
        return self.visit(ctx.predicate())

    def visitBetweenPredicate(self, ctx: MySqlBaseParser.BetweenPredicateContext):
        log('visitBetweenPredicate')
        bp = BetweenPredicate()
        bp.predicate1 = self.visit(ctx.predicate(0))
        bp.NOT = ctx.NOT() is not None
        bp.predicate2 = self.visit(ctx.predicate(1))
        bp.predicate3 = self.visit(ctx.predicate(2))
        return bp

    def visitNotExpression(self, ctx: MySqlBaseParser.NotExpressionContext):
        log('visitNotExpression')
        np = NotExpression()
        np.expression = self.visit(ctx.expression())
        return np

    def visitLogicalExpression(self, ctx: MySqlBaseParser.LogicalExpressionContext):
        log('visitLogicalExpression')
        lp = LogicalExpression()
        lp.expression1 = self.visit(ctx.expression(0))
        lp.logicalOperator = self.visit(ctx.logicalOperator())
        lp.expression2 = self.visit(ctx.expression(1))
        return lp

    def visitLikePredicate(self, ctx: MySqlBaseParser.LikePredicateContext):
        log('visitLikePredicate')
        lp = LikePredicate()
        lp.predicate1 = self.visit(ctx.predicate(0))
        lp.NOT = ctx.NOT() is not None
        lp.predicate2 = self.visit(ctx.predicate(1))
        return lp

    def visitNullPredicate(self, ctx: MySqlBaseParser.NullPredicateContext):
        log('visitNullPredicate')
        np = NullPredicate()
        np.predicate = self.visit(ctx.predicate())
        np.NOT = ctx.NOT() is not None
        return np

    def visitPredicate(self, ctx: MySqlBaseParser.PredicateContext):
        log('visitPredicate')
        if ctx.fullColumnName() is not None:
            return self.visit(ctx.fullColumnName())
        elif ctx.constant() is not None:
            return self.visit(ctx.constant())
        elif ctx.functionCall() is not None:
            return self.visit(ctx.functionCall())

    def visitSpecificFunctionCall(self, ctx: MySqlBaseParser.SpecificFunctionCallContext):
        log('visitSpecificFunctionCall')
        return self.visit(ctx.specificFunction())

    def visitSimpleFunctionCall(self, ctx: MySqlBaseParser.SimpleFunctionCallContext):
        log('visitSpecificFunctionCall')
        fc = FunctionCall()
        fc.function_name = ctx.getText().replace('(', '').replace(')')
        return fc

    def visitScalarFunctionCall(self, ctx: MySqlBaseParser.ScalarFunctionCallContext):
        log('visitScalarFunctionCall')
        fc = FunctionCall()
        fc.function_name = self.visit(ctx.scalarFunctionName())
        if ctx.functionArgs() is not None:
            fc.args_list = self.visit(ctx.functionArgs())
        return fc

    def visitScalarFunctionName(self, ctx: MySqlBaseParser.ScalarFunctionNameContext):
        log('visitScalarFunctionCall')
        return ctx.getText()

    def visitFunctionArgs(self, ctx: MySqlBaseParser.FunctionArgsContext):
        log('visitFunctionArgs')
        fas = ctx.functionArg()
        args_list = []
        for fa in fas:
            args_list.append(self.visit(fa))
        return args_list

    def visitFunctionArg(self, ctx: MySqlBaseParser.FunctionArgContext):
        log('visitFunctionArg')
        if ctx.constant() is not None:
            return self.visit(ctx.constant())
        elif ctx.fullColumnName() is not None:
            return self.visit(ctx.fullColumnName())
        elif ctx.functionCall() is not None:
            return self.visit(ctx.functionCall())

    def visitComparisonOperator(self, ctx: MySqlBaseParser.ComparisonOperatorContext):
        log('visitComparisonOperator')
        return ctx.getText()

    def visitLogicalOperator(self, ctx: MySqlBaseParser.LogicalOperatorContext):
        log('visitLogicalOperator')
        return ctx.getText()

    def visitKeywordsCanBeId(self, ctx: MySqlBaseParser.KeywordsCanBeIdContext):
        log('visitKeywordsCanBeId')
        return ctx.getText()

    def visitConstant(self, ctx: MySqlBaseParser.ConstantContext):
        log('visitConstant')
        if ctx.bitString() is not None:
            return self.visit(ctx.bitString())
        elif ctx.realLiteral() is not None:
            return self.visit(ctx.realLiteral())
        elif ctx.decimalLiteral() is not None:
            return self.visit(ctx.decimalLiteral())
        elif ctx.stringLiteral() is not None:
            return self.visit(ctx.stringLiteral())
        elif ctx.booleanLiteral() is not None:
            return self.visit(ctx.booleanLiteral())
        elif ctx.parameterMarker() is not None:
            return self.visit(ctx.parameterMarker())
        elif ctx.hexadecimalLiteral() is not None:
            return self.visit(ctx.hexadecimalLiteral())
        elif ctx.nullLiteral() is not None:
            return self.visit(ctx.nullLiteral())
        else:
            print('constant unknown')

    def visitBitString(self, ctx: MySqlBaseParser.BitStringContext):
        log('visitBitString')
        return BitString(ctx.getText())

    def visitRealLiteral(self, ctx: MySqlBaseParser.RealLiteralContext):
        log('visitRealLiteral')
        return RealLiteral(ctx.getText())

    def visitBooleanLiteral(self, ctx: MySqlBaseParser.BooleanLiteralContext):
        log('visitBooleanLiteral')
        return BooleanLiteral(ctx.getText())

    def visitStringLiteral(self, ctx: MySqlBaseParser.StringLiteralContext):
        log('visitStringLiteral')
        return StringLiteral(ctx.getText())

    def visitDecimalLiteral(self, ctx: MySqlBaseParser.DecimalLiteralContext):
        log('visitDecimalLiteral')
        return DecimalLiteral(ctx.getText())

    def visitHexadecimalLiteral(self, ctx: MySqlBaseParser.HexadecimalLiteralContext):
        log('visitHexadecimalLiteral')
        return HexadecimalLiteral(ctx.getText())

    def visitNullLiteral(self, ctx: MySqlBaseParser.NullLiteralContext):
        log('visitNullLiteral')
        return NullLiteral()

    def visitOrderByClause(self, ctx: MySqlBaseParser.OrderByClauseContext):
        log('visitOrderByClause')
        obc = OrderByClause()
        order_by_exprs = []
        obes = ctx.orderByExpression()
        for i in range(len(obes)):
            order_by_exprs.append(self.visitOrderByExpression(obes[i]))
        obc.orderByExpressions = order_by_exprs
        return obc

    def visitOrderByExpression(self, ctx: MySqlBaseParser.OrderByExpressionContext):
        log('visitOrderByExpression')
        obe = OrderByExpression()
        obe.expression = self.visit(ctx.expression())
        if ctx.ASC() is None and ctx.DESC() is None:
            obe.ASC = True
        elif ctx.ASC() is not None:
            obe.ASC = True
        elif ctx.DESC() is not None:
            obe.ASC = False
        return obe

    def visitLimitClause(self, ctx: MySqlBaseParser.LimitClauseContext):
        log('visitLimitClause')
        lc = LimitClause()
        if ctx.COMMA() is not None:
            lc.offset = self.visitDecimalLiteral(ctx.decimalLiteral(0))
            lc.limit = self.visitDecimalLiteral(ctx.decimalLiteral(1))
        else:
            lc.limit = self.visitDecimalLiteral(ctx.decimalLiteral(0))
        return lc

    def visitLockClause(self, ctx: MySqlBaseParser.LockClauseContext):
        log('visitLockClause')
        lc = LockClause()
        if ctx.UPDATE() is not None:
            lc.forUpdate = True
        else:
            lc.forUpdate = False
        return lc

    def visitParameterMarker(self, ctx: MySqlBaseParser.ParameterMarkerContext):
        log('visitParameterMarker')
        self.parameter_marker_index += 1
        name = None
        if ctx.PERCENT_NAME_() is not None:
            name = ctx.PERCENT_NAME_().getText()[2:-2]
        pm = ParameterMarker(self.parameter_marker_index, name, ctx.getText())
        return pm


class MysqlOutputVisitor:

    def visitSelectStatement(self, statement, output):
        if not isinstance(statement, SelectStatement):
            raise TypeError('select statement type error. ' + type(statement).__name__)
        if not isinstance(output, list):
            raise TypeError('select statement output type error. ' + type(output).__name__)

        output.append("SELECT")
        if len(statement.selectElements) == 0:
            output.append(" ")
            output.append("*")
        else:
            for selectElement in statement.selectElements:
                self.visitSelectElement(selectElement, output)
        output.append(" ")
        output.append("FROM")
        self.visitTableSource(statement.tableSource, output)

        if statement.where is not None:
            self.visitWhere(statement.where, output)
        if statement.orderBy is not None:
            self.visitOrderBy(statement.orderBy, output)
        if statement.limit is not None:
            self.visitLimit(statement.limit, output)
        if statement.lock is not None:
            self.visitLock(statement.lock, output)

    def visitSelectElement(self, select_element, output):
        if not isinstance(output, list):
            raise TypeError('select_element output type error. ' + type(output).__name__)
        if isinstance(select_element, FullColumnName):
            self.visitFullColumnName(select_element, output)
        else:
            output.append(" ")
            output.append(select_element)

    def visitWhere(self, where, output):
        if not isinstance(where, WhereClause):
            raise TypeError("where type error. " + type(where).__name__)
        if not isinstance(output, list):
            raise TypeError('where output type error. ' + type(output).__name__)
        output.append(" ")
        output.append("WHERE")
        self.visitExpression(where.expression, output)

    def visitOrderBy(self, order_by, output):
        if not isinstance(order_by, OrderByClause):
            raise TypeError("order_by type error. " + type(order_by).__name__)
        if not isinstance(output, list):
            raise TypeError('order_by output type error. ' + type(output).__name__)
        output.append(" ")
        output.append("ORDER")
        output.append(" ")
        output.append("BY")
        for obe_idx, obe in enumerate(order_by.orderByExpressions):
            if obe_idx > 0:
                output.append(",")
            self.visitOrderByExpression(obe, output)

    def visitOrderByExpression(self, order_by_expression, output):
        if not isinstance(order_by_expression, OrderByExpression):
            raise TypeError("order_by_expression type error. " + type(order_by_expression).__name__)
        if not isinstance(output, list):
            raise TypeError('order_by_expression output type error. ' + type(output).__name__)
        self.visitExpression(order_by_expression.expression, output)
        output.append(" ")
        if order_by_expression.ASC:
            output.append("ASC")
        else:
            output.append("DESC")

    def visitLimit(self, limit, output):
        if not isinstance(limit, LimitClause):
            raise TypeError("limit type error. " + type(limit).__name__)
        if not isinstance(output, list):
            raise TypeError('limit output type error. ' + type(output).__name__)
        output.append(" ")
        output.append("LIMIT")
        if limit.offset is not None:
            self.visitConstant(limit.offset, output)
            output.append(",")
        self.visitConstant(limit.limit, output)

    def visitLock(self, lock, output):
        if not isinstance(lock, LockClause):
            raise TypeError("lock type error. " + type(lock).__name__)
        if not isinstance(output, list):
            raise TypeError('lock output type error. ' + type(output).__name__)

        output.append(" ")
        if lock.forUpdate:
            output.append("FOR UPDATE")
        else:
            output.append("LOCK IN SHARE MODE")

    def visitInsertStatement(self, statement, output):
        if not isinstance(statement, InsertStatement):
            raise TypeError('insert statement type error. ' + type(statement).__name__)
        if not isinstance(output, list):
            raise TypeError('insert statement output type error. ' + type(output).__name__)
        output.append('INSERT')
        if statement.ignore:
            output.append(" ")
            output.append('IGNORE')
        output.append(" ")
        output.append('INTO')
        self.visitTableSource(statement.tableSource, output)
        self.visitColumns(statement.columns, output)
        self.visitValueList(statement.value_list, output)

    def visitUpdateStatement(self, statement, output):
        if not isinstance(statement, UpdateStatement):
            raise TypeError('update statement type error. ' + type(statement).__name__)
        if not isinstance(output, list):
            raise TypeError('update statement output type error. ' + type(output).__name__)

        output.append("UPDATE")
        self.visitTableSource(statement.tableSource, output)
        output.append(" ")
        output.append("SET")
        for ele_idx, ele in enumerate(statement.updatedElement):
            if ele_idx > 0:
                output.append(",")
            self.visitUpdatedElement(ele, output)
        if statement.where is not None:
            self.visitWhere(statement.where, output)
        if statement.orderBy is not None:
            self.visitOrderBy(statement.orderBy, output)
        if statement.limit is not None:
            self.visitLimit(statement.limit, output)

    def visitUpdatedElement(self, updated_element, output):
        if not isinstance(updated_element, UpdateElement):
            raise TypeError("updated_element type error. " + type(updated_element).__name__)
        if not isinstance(output, list):
            raise TypeError('updated_element output type error. ' + type(output).__name__)
        self.visitFullColumnName(updated_element.column, output)
        output.append(" ")
        output.append("=")
        if isinstance(updated_element.value, str):
            output.append(" ")
            output.append(updated_element.value)
        else:
            self.visitExpression(updated_element.value, output)

    def visitDeleteStatement(self, statement, output):
        if not isinstance(statement, DeleteStatement):
            raise TypeError('delete statement type error. ' + type(statement).__name__)
        if not isinstance(output, list):
            raise TypeError('delete statement output type error. ' + type(output).__name__)
        output.append("DELETE")
        if statement.tableAlias is not None:
            output.append(" ")
            output.append(statement.tableAlias)
        output.append(" ")
        output.append("FROM")
        self.visitTableSource(statement.tableSource, output)
        if statement.where is not None:
            self.visitWhere(statement.where, output)
        if statement.orderBy is not None:
            self.visitOrderBy(statement.orderBy, output)
        if statement.limit is not None:
            output.append(" ")
            output.append("LIMIT")
            self.visitConstant(statement.limit, output)

    def visitTableSource(self, table_source, output):
        if not isinstance(table_source, TableSource):
            raise TypeError('table_source type error. ' + type(table_source).__name__)
        if not isinstance(output, list):
            raise TypeError('table_source output type error. ' + type(output).__name__)
        output.append(" ")
        output.append(table_source.tableName)
        if table_source.tableAlias is not None:
            output.append(" ")
            output.append(table_source.tableAlias)

    def visitColumns(self, columns, output):
        if not isinstance(output, list):
            raise TypeError('columns output type error. ' + type(output).__name__)
        if columns is not None and len(columns) > 0:
            output.append(" ")
            output.append("(")
            for col_idx, col in enumerate(columns):
                if col_idx > 0:
                    output.append(',')
                output.append(" ")
                output.append(col)
            output.append(")")

    def visitValueList(self, value_list, output):
        if not isinstance(output, list):
            raise TypeError('value_list output type error. ' + type(output).__name__)
        output.append(" ")
        output.append('VALUES')
        for exprs_idx, expressions in enumerate(value_list):
            if exprs_idx > 0:
                output.append(",")
            output.append(" ")
            output.append("(")
            for expr_idx, expression in enumerate(expressions):
                if expr_idx > 0:
                    output.append(",")
                self.visitExpression(expression, output)
            output.append(")")

    def visitExpression(self, expression, output):
        self.visitPredicate(expression, output)

    def visitPredicate(self, predicate, output):
        if isinstance(predicate, InPredicate):
            self.visitPredicate(predicate.predicate, output)
            if predicate.NOT:
                output.append(" ")
                output.append('NOT')
            output.append(" ")
            output.append("IN")
            output.append(" ")
            output.append('(')
            self.visitExpressions(predicate.expressions, output)
            output.append(" ")
            output.append(")")
        elif isinstance(predicate, ComparisonPredicate):
            self.visitPredicate(predicate.predicate1, output)
            output.append(" ")
            output.append(predicate.comparisonOperator)
            self.visitPredicate(predicate.predicate2, output)
        elif isinstance(predicate, NullPredicate):
            self.visitPredicate(predicate.predicate, output)
            output.append(" ")
            output.append("IS")
            if predicate.NOT:
                output.append(" ")
                output.append("NOT")
            output.append(" ")
            output.append("NULL")
        elif isinstance(predicate, BetweenPredicate):
            self.visitPredicate(predicate.predicate1, output)
            if predicate.NOT:
                output.append(" ")
                output.append("NOT")
            output.append(" ")
            output.append("BETWEEN")
            self.visitPredicate(predicate.predicate2, output)
            output.append(" ")
            output.append("AND")
            self.visitPredicate(predicate.predicate3, output)
        elif isinstance(predicate, LikePredicate):
            self.visitPredicate(predicate.predicate1, output)
            if predicate.NOT:
                output.append(" ")
                output.append("NOT")
            output.append(" ")
            output.append("LIKE")
            self.visitPredicate(predicate.predicate2, output)
        elif isinstance(predicate, NotExpression):
            output.append(" ")
            output.append("NOT")
            self.visitExpression(predicate.expression, output)
        elif isinstance(predicate, LogicalExpression):
            self.visitExpression(predicate.expression1, output)
            output.append(" ")
            output.append(predicate.logicalOperator.upper())
            self.visitExpression(predicate.expression2, output)
        elif isinstance(predicate, PredicateAtom):
            if isinstance(predicate, FullColumnName):
                self.visitFullColumnName(predicate, output)
            elif isinstance(predicate, Constant):
                self.visitConstant(predicate, output)
            elif isinstance(predicate, FunctionCall):
                self.visitFunctionCall(predicate, output)
            else:
                print('predicate atom unknown')
        else:
            print('predicate unknown')

    def visitExpressions(self, expressions, output):
        for expr_idx, expression in enumerate(expressions):
            if expr_idx > 0:
                output.append(",")
            self.visitExpression(expression, output)

    def visitFullColumnName(self, predicate, output):
        output.append(" ")
        if predicate.owner is not None:
            output.append(predicate.owner + predicate.value)
        else:
            output.append(predicate.value)

    def visitConstant(self, val, output):
        output.append(" ")
        if isinstance(val, BitString):
            output.append(str(val.value))
        elif isinstance(val, RealLiteral):
            output.append(str(val.value))
        elif isinstance(val, DecimalLiteral):
            output.append(str(val.value))
        elif isinstance(val, BooleanLiteral):
            output.append(str(val.value).upper())
        elif isinstance(val, ParameterMarker):
            self.visitParameterMarker(val, output)
        elif isinstance(val, StringLiteral):
            output.append('\'' + val.value + '\'')
        elif isinstance(val, HexadecimalLiteral):
            output.append(str(val.value))
        elif isinstance(val, NullLiteral):
            output.append('NULL')
        else:
            print('constant unknown, ' + type(val).__name__)

    def visitParameterMarker(self, parameter_marker, output):
        output.append(parameter_marker.value)

    def visitFunctionCall(self, function_call, output):
        if not isinstance(function_call, FunctionCall):
            raise TypeError('function_call type error. ' + type(function_call).__name__)
        if not isinstance(output, list):
            raise TypeError('function_call output type error. ' + type(output).__name__)
        output.append(" ")
        output.append(function_call.function_name)
        output.append("(")
        if function_call.args_list is not None:
            for arg_idx, arg in enumerate(function_call.args_list):
                if arg_idx > 0:
                    output.append(",")
                if isinstance(arg, Constant):
                    self.visitConstant(arg, output)
                elif isinstance(arg, FullColumnName):
                    self.visitConstant(arg, output)
                elif isinstance(arg, FunctionCall):
                    self.visitFunctionCall(arg, output)
        output.append(")")



# Generated from MySqlBaseParser.g4 by ANTLR 4.9.2
from antlr4 import *
if __name__ is not None and "." in __name__:
    from .MySqlBaseParser import MySqlBaseParser
else:
    from MySqlBaseParser import MySqlBaseParser

# This class defines a complete listener for a parse tree produced by MySqlBaseParser.
class MySqlBaseParserListener(ParseTreeListener):

    # Enter a parse tree produced by MySqlBaseParser#root.
    def enterRoot(self, ctx:MySqlBaseParser.RootContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#root.
    def exitRoot(self, ctx:MySqlBaseParser.RootContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#sqlStatements.
    def enterSqlStatements(self, ctx:MySqlBaseParser.SqlStatementsContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#sqlStatements.
    def exitSqlStatements(self, ctx:MySqlBaseParser.SqlStatementsContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#sqlStatement.
    def enterSqlStatement(self, ctx:MySqlBaseParser.SqlStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#sqlStatement.
    def exitSqlStatement(self, ctx:MySqlBaseParser.SqlStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#emptyStatement.
    def enterEmptyStatement(self, ctx:MySqlBaseParser.EmptyStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#emptyStatement.
    def exitEmptyStatement(self, ctx:MySqlBaseParser.EmptyStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#dmlStatement.
    def enterDmlStatement(self, ctx:MySqlBaseParser.DmlStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#dmlStatement.
    def exitDmlStatement(self, ctx:MySqlBaseParser.DmlStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#selectStatement.
    def enterSelectStatement(self, ctx:MySqlBaseParser.SelectStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#selectStatement.
    def exitSelectStatement(self, ctx:MySqlBaseParser.SelectStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#insertStatement.
    def enterInsertStatement(self, ctx:MySqlBaseParser.InsertStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#insertStatement.
    def exitInsertStatement(self, ctx:MySqlBaseParser.InsertStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#updateStatement.
    def enterUpdateStatement(self, ctx:MySqlBaseParser.UpdateStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#updateStatement.
    def exitUpdateStatement(self, ctx:MySqlBaseParser.UpdateStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#deleteStatement.
    def enterDeleteStatement(self, ctx:MySqlBaseParser.DeleteStatementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#deleteStatement.
    def exitDeleteStatement(self, ctx:MySqlBaseParser.DeleteStatementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#insertStatementValue.
    def enterInsertStatementValue(self, ctx:MySqlBaseParser.InsertStatementValueContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#insertStatementValue.
    def exitInsertStatementValue(self, ctx:MySqlBaseParser.InsertStatementValueContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#selectElements.
    def enterSelectElements(self, ctx:MySqlBaseParser.SelectElementsContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#selectElements.
    def exitSelectElements(self, ctx:MySqlBaseParser.SelectElementsContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#selectStarElement.
    def enterSelectStarElement(self, ctx:MySqlBaseParser.SelectStarElementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#selectStarElement.
    def exitSelectStarElement(self, ctx:MySqlBaseParser.SelectStarElementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#selectColumnElement.
    def enterSelectColumnElement(self, ctx:MySqlBaseParser.SelectColumnElementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#selectColumnElement.
    def exitSelectColumnElement(self, ctx:MySqlBaseParser.SelectColumnElementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#tableSource.
    def enterTableSource(self, ctx:MySqlBaseParser.TableSourceContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#tableSource.
    def exitTableSource(self, ctx:MySqlBaseParser.TableSourceContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#tableName.
    def enterTableName(self, ctx:MySqlBaseParser.TableNameContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#tableName.
    def exitTableName(self, ctx:MySqlBaseParser.TableNameContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#tableAlias.
    def enterTableAlias(self, ctx:MySqlBaseParser.TableAliasContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#tableAlias.
    def exitTableAlias(self, ctx:MySqlBaseParser.TableAliasContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#fullId.
    def enterFullId(self, ctx:MySqlBaseParser.FullIdContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#fullId.
    def exitFullId(self, ctx:MySqlBaseParser.FullIdContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#fullColumnName.
    def enterFullColumnName(self, ctx:MySqlBaseParser.FullColumnNameContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#fullColumnName.
    def exitFullColumnName(self, ctx:MySqlBaseParser.FullColumnNameContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#dottedId.
    def enterDottedId(self, ctx:MySqlBaseParser.DottedIdContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#dottedId.
    def exitDottedId(self, ctx:MySqlBaseParser.DottedIdContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#uidList.
    def enterUidList(self, ctx:MySqlBaseParser.UidListContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#uidList.
    def exitUidList(self, ctx:MySqlBaseParser.UidListContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#uid.
    def enterUid(self, ctx:MySqlBaseParser.UidContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#uid.
    def exitUid(self, ctx:MySqlBaseParser.UidContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#updatedElement.
    def enterUpdatedElement(self, ctx:MySqlBaseParser.UpdatedElementContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#updatedElement.
    def exitUpdatedElement(self, ctx:MySqlBaseParser.UpdatedElementContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#whereClause.
    def enterWhereClause(self, ctx:MySqlBaseParser.WhereClauseContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#whereClause.
    def exitWhereClause(self, ctx:MySqlBaseParser.WhereClauseContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#expressions.
    def enterExpressions(self, ctx:MySqlBaseParser.ExpressionsContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#expressions.
    def exitExpressions(self, ctx:MySqlBaseParser.ExpressionsContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#comparisonPredicate.
    def enterComparisonPredicate(self, ctx:MySqlBaseParser.ComparisonPredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#comparisonPredicate.
    def exitComparisonPredicate(self, ctx:MySqlBaseParser.ComparisonPredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#expressionAtomPredicate.
    def enterExpressionAtomPredicate(self, ctx:MySqlBaseParser.ExpressionAtomPredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#expressionAtomPredicate.
    def exitExpressionAtomPredicate(self, ctx:MySqlBaseParser.ExpressionAtomPredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#inPredicate.
    def enterInPredicate(self, ctx:MySqlBaseParser.InPredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#inPredicate.
    def exitInPredicate(self, ctx:MySqlBaseParser.InPredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#betweenPredicate.
    def enterBetweenPredicate(self, ctx:MySqlBaseParser.BetweenPredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#betweenPredicate.
    def exitBetweenPredicate(self, ctx:MySqlBaseParser.BetweenPredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#notExpression.
    def enterNotExpression(self, ctx:MySqlBaseParser.NotExpressionContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#notExpression.
    def exitNotExpression(self, ctx:MySqlBaseParser.NotExpressionContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#likePredicate.
    def enterLikePredicate(self, ctx:MySqlBaseParser.LikePredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#likePredicate.
    def exitLikePredicate(self, ctx:MySqlBaseParser.LikePredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#nullPredicate.
    def enterNullPredicate(self, ctx:MySqlBaseParser.NullPredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#nullPredicate.
    def exitNullPredicate(self, ctx:MySqlBaseParser.NullPredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#logicalExpression.
    def enterLogicalExpression(self, ctx:MySqlBaseParser.LogicalExpressionContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#logicalExpression.
    def exitLogicalExpression(self, ctx:MySqlBaseParser.LogicalExpressionContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#predicate.
    def enterPredicate(self, ctx:MySqlBaseParser.PredicateContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#predicate.
    def exitPredicate(self, ctx:MySqlBaseParser.PredicateContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#specificFunctionCall.
    def enterSpecificFunctionCall(self, ctx:MySqlBaseParser.SpecificFunctionCallContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#specificFunctionCall.
    def exitSpecificFunctionCall(self, ctx:MySqlBaseParser.SpecificFunctionCallContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#scalarFunctionCall.
    def enterScalarFunctionCall(self, ctx:MySqlBaseParser.ScalarFunctionCallContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#scalarFunctionCall.
    def exitScalarFunctionCall(self, ctx:MySqlBaseParser.ScalarFunctionCallContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#scalarFunctionName.
    def enterScalarFunctionName(self, ctx:MySqlBaseParser.ScalarFunctionNameContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#scalarFunctionName.
    def exitScalarFunctionName(self, ctx:MySqlBaseParser.ScalarFunctionNameContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#simpleFunctionCall.
    def enterSimpleFunctionCall(self, ctx:MySqlBaseParser.SimpleFunctionCallContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#simpleFunctionCall.
    def exitSimpleFunctionCall(self, ctx:MySqlBaseParser.SimpleFunctionCallContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#functionArgs.
    def enterFunctionArgs(self, ctx:MySqlBaseParser.FunctionArgsContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#functionArgs.
    def exitFunctionArgs(self, ctx:MySqlBaseParser.FunctionArgsContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#functionArg.
    def enterFunctionArg(self, ctx:MySqlBaseParser.FunctionArgContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#functionArg.
    def exitFunctionArg(self, ctx:MySqlBaseParser.FunctionArgContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#functionNameBase.
    def enterFunctionNameBase(self, ctx:MySqlBaseParser.FunctionNameBaseContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#functionNameBase.
    def exitFunctionNameBase(self, ctx:MySqlBaseParser.FunctionNameBaseContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#comparisonOperator.
    def enterComparisonOperator(self, ctx:MySqlBaseParser.ComparisonOperatorContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#comparisonOperator.
    def exitComparisonOperator(self, ctx:MySqlBaseParser.ComparisonOperatorContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#logicalOperator.
    def enterLogicalOperator(self, ctx:MySqlBaseParser.LogicalOperatorContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#logicalOperator.
    def exitLogicalOperator(self, ctx:MySqlBaseParser.LogicalOperatorContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#keywordsCanBeId.
    def enterKeywordsCanBeId(self, ctx:MySqlBaseParser.KeywordsCanBeIdContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#keywordsCanBeId.
    def exitKeywordsCanBeId(self, ctx:MySqlBaseParser.KeywordsCanBeIdContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#constant.
    def enterConstant(self, ctx:MySqlBaseParser.ConstantContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#constant.
    def exitConstant(self, ctx:MySqlBaseParser.ConstantContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#nullLiteral.
    def enterNullLiteral(self, ctx:MySqlBaseParser.NullLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#nullLiteral.
    def exitNullLiteral(self, ctx:MySqlBaseParser.NullLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#realLiteral.
    def enterRealLiteral(self, ctx:MySqlBaseParser.RealLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#realLiteral.
    def exitRealLiteral(self, ctx:MySqlBaseParser.RealLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#bitString.
    def enterBitString(self, ctx:MySqlBaseParser.BitStringContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#bitString.
    def exitBitString(self, ctx:MySqlBaseParser.BitStringContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#booleanLiteral.
    def enterBooleanLiteral(self, ctx:MySqlBaseParser.BooleanLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#booleanLiteral.
    def exitBooleanLiteral(self, ctx:MySqlBaseParser.BooleanLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#stringLiteral.
    def enterStringLiteral(self, ctx:MySqlBaseParser.StringLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#stringLiteral.
    def exitStringLiteral(self, ctx:MySqlBaseParser.StringLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#decimalLiteral.
    def enterDecimalLiteral(self, ctx:MySqlBaseParser.DecimalLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#decimalLiteral.
    def exitDecimalLiteral(self, ctx:MySqlBaseParser.DecimalLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#hexadecimalLiteral.
    def enterHexadecimalLiteral(self, ctx:MySqlBaseParser.HexadecimalLiteralContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#hexadecimalLiteral.
    def exitHexadecimalLiteral(self, ctx:MySqlBaseParser.HexadecimalLiteralContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#orderByClause.
    def enterOrderByClause(self, ctx:MySqlBaseParser.OrderByClauseContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#orderByClause.
    def exitOrderByClause(self, ctx:MySqlBaseParser.OrderByClauseContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#orderByExpression.
    def enterOrderByExpression(self, ctx:MySqlBaseParser.OrderByExpressionContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#orderByExpression.
    def exitOrderByExpression(self, ctx:MySqlBaseParser.OrderByExpressionContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#limitClause.
    def enterLimitClause(self, ctx:MySqlBaseParser.LimitClauseContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#limitClause.
    def exitLimitClause(self, ctx:MySqlBaseParser.LimitClauseContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#lockClause.
    def enterLockClause(self, ctx:MySqlBaseParser.LockClauseContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#lockClause.
    def exitLockClause(self, ctx:MySqlBaseParser.LockClauseContext):
        pass


    # Enter a parse tree produced by MySqlBaseParser#parameterMarker.
    def enterParameterMarker(self, ctx:MySqlBaseParser.ParameterMarkerContext):
        pass

    # Exit a parse tree produced by MySqlBaseParser#parameterMarker.
    def exitParameterMarker(self, ctx:MySqlBaseParser.ParameterMarkerContext):
        pass



del MySqlBaseParser


#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @author jsbxyyx
import os

import setuptools

VERSION = "0.1.1"

base_dir = os.path.dirname(os.path.abspath(__file__))

with open(os.path.join(base_dir, "README.md"), "r") as f:
    long_description = f.read()

with open(os.path.join(base_dir, "requirements.txt"), "r") as f:
    install_requires = f.read().splitlines()

setuptools.setup(
    name="antlr4-mysql",
    version=VERSION,
    author="jsbxyyx",
    author_email="jsbxyyx@163.com",
    license="Apache License 2.0",
    description="antlr4-mysql",
    long_description=long_description,
    long_description_content_type="text/markdown",
    install_requires=install_requires,
    url="https://github.com/jsbxyyx/antlr4-mysql",
    packages=setuptools.find_packages(),
    include_package_data=True,
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ]
)



#!/usr/bin/env python3
# -*- coding:utf-8 -*-
# @author jsbxyyx
if __name__ is not None and "." in __name__:
    from .mysql_base import MySqlBase, MysqlStatementVisitor, MysqlOutputVisitor
else:
    from am.mysql_base import MySqlBase, MysqlStatementVisitor, MysqlOutputVisitor


def test_sql():
    select_sql = """
        select * from test where name = ? and id = ?;
        select * from test where name = ? and id = ? order by t.id asc, t.name desc limit 10, 0 for update;
        """
    ss = MySqlBase.parserSQLStatement(select_sql)
    for idx in range(len(ss)):
        s = MysqlStatementVisitor().visit(ss[idx])
        output = list()
        MysqlOutputVisitor().visitSelectStatement(s, output)
        string = ''.join(output)
        print(string)
        print('*' * 100)

    insert_sql = """
        insert into test(id, name) values(?,?), (?,?), (?, 1), (?, '1'), (?, 1.1);
        insert into test(id, name) values(?,now());
        insert into test(id, name) values(null,now());
        """
    ss = MySqlBase.parserSQLStatement(insert_sql)
    for idx in range(len(ss)):
        s = MysqlStatementVisitor().visit(ss[idx])
        output = list()
        MysqlOutputVisitor().visitInsertStatement(s, output)
        string = ''.join(output)
        print(string)
        print('*' * 100)

    update_sql = """
        update test t set t.id = ?, name = ? where t.id in (?,?) and t.like like '%1%';
        update test t set t.id = ?, name = ? where t.id in (?,?) and t.id is null;
        update test t set t.id = ?, name = ? where t.id in (?,?) and t.like like '%1%' order by t.id, t.name desc limit 1;
        """
    ss = MySqlBase.parserSQLStatement(update_sql)
    for idx in range(len(ss)):
        s = MysqlStatementVisitor().visit(ss[idx])
        output = list()
        MysqlOutputVisitor().visitUpdateStatement(s, output)
        string = ''.join(output)
        print(string)
        print('*' * 100)

    delete_sql = """
        delete t from test t where t.id = ? and t.name in (?,?);
        delete t from test t where t.id between 10 and 30 and t.name = '';
        delete t from test t where t.id between 10 and 30 and t.name = '' order by t1.name desc limit 1;
        delete t from test t where t.name = 0x1 and t.name = x'11' and t.name = b'1' order by t1.name desc limit 1;
        delete t from test t where t.name = format(1.23, 1) and t.created = date_format( now(), '%Y%m%d');
        """
    ss = MySqlBase.parserSQLStatement(delete_sql)
    for idx in range(len(ss)):
        s = MysqlStatementVisitor().visit(ss[idx])
        output = list()
        MysqlOutputVisitor().visitDeleteStatement(s, output)
        string = ''.join(output)
        print(string)
        print('*' * 100)

    sql = """
        insert into test values(%(user_id)s, %(count)s);
        """
    ss = MySqlBase.parserSQLStatement(sql)
    for idx in range(len(ss)):
        s = MysqlStatementVisitor().visit(ss[idx])
        output = list()
        MysqlOutputVisitor().visitInsertStatement(s, output)
        string = ''.join(output)
        print(string)
        print('*' * 100)

    print()


if __name__ == '__main__':
    test_sql()
